# $Id: mt-counter.pl 735 2007-08-09 08:08:13Z endo $

package MT::Plugin::Counter;

use strict;
use MT::Plugin;

@MT::Plugin::Counter::ISA = qw(MT::Plugin);

use vars qw($PLUGIN_NAME $VERSION);
$PLUGIN_NAME = 'Counter';
$VERSION = '0.60';

use MT;
my $plugin = new MT::Plugin::Counter({
    name => $PLUGIN_NAME,
    version => $VERSION,
    description => "This plugin allows you to use a counter in a template. You can compare the value of the counter with a specific number or can check whether it contains an odd/even number.",
    doc_link => 'http://labs.m-logic.jp/plugins/mt-counter/docs/mt-counter.html',
    author_name => 'M-Logic, Inc.',
    author_link => 'http://labs.m-logic.jp/',
});

MT->add_plugin($plugin);

require MT::Template::Context;
MT::Template::Context->add_container_tag('Counter' => sub{&hdlr_counter;});
MT::Template::Context->add_tag('CounterSetValue' => sub{&hdlr_counter_set_value;});
MT::Template::Context->add_tag('CounterValue' => sub{&hdlr_counter_value;});
MT::Template::Context->add_tag('CounterPlus' => sub{&hdlr_counter_plus;});
MT::Template::Context->add_tag('CounterMinus' => sub{&hdlr_counter_minus;});
MT::Template::Context->add_conditional_tag('CounterIfEqual' => sub{&hdlr_counter_if_equal;});
MT::Template::Context->add_conditional_tag('CounterIfLess' => sub{&hdlr_counter_if_less;});
MT::Template::Context->add_conditional_tag('CounterIfMore' => sub{&hdlr_counter_if_more;});
MT::Template::Context->add_conditional_tag('CounterIfOdd' => sub{&hdlr_counter_if_odd;});
MT::Template::Context->add_conditional_tag('CounterIfEven' => sub{&hdlr_counter_if_even;});
MT::Template::Context->add_conditional_tag('CounterIfMod' => sub{&hdlr_counter_if_mod;});


sub hdlr_counter {
    my ($ctx, $args, $cond) = @_;
    my $builder = $ctx->stash('builder');
    my $tokens = $ctx->stash('tokens');

    $ctx->stash('counter_count', 0);
    defined(my $text = $builder->build($ctx, $tokens, $cond))
        || return $ctx->error($ctx->errstr);

    return $text;

}

sub hdlr_counter_set_value {
    my ($ctx, $args) = @_;
    my $value = $args->{value} || 0;

    $ctx->stash('counter_count', $value);

    return '';
}

sub hdlr_counter_value {
    my ($ctx, $args) = @_;
    my $count = $ctx->stash('counter_count') || 0;

    return "$count";
}

sub hdlr_counter_plus {
    my ($ctx, $args) = @_;
    my $count = $ctx->stash('counter_count');
    
    if (defined($count)) {
        $ctx->stash('counter_count', ++$count);
    }

    return '';
}

sub hdlr_counter_minus {
    my ($ctx, $args) = @_;
    my $count = $ctx->stash('counter_count');
    
    if (defined($count)) {
        $ctx->stash('counter_count', --$count);
    }

    return '';
}

sub hdlr_counter_if_equal {
    my ($ctx, $args, $cond) = @_;
    my $value = $args->{value} || 0;
    my $count = $ctx->stash('counter_count');
    
    return (defined($count) && $value == $count);
}

sub hdlr_counter_if_less {
    my ($ctx, $args, $cond) = @_;
    my $value = $args->{value} || 0;
    my $count = $ctx->stash('counter_count');
    
    return (defined($count) && $value > $count);
}

sub hdlr_counter_if_more {
    my ($ctx, $args, $cond) = @_;
    my $value = $args->{value} || 0;
    my $count = $ctx->stash('counter_count');
    
    return (defined($count) && $value < $count);
}

sub hdlr_counter_if_odd {
    my ($ctx, $args, $cond) = @_;
    my $count = $ctx->stash('counter_count');
    
    return (defined($count) && ($count % 2 == 1));
}

sub hdlr_counter_if_even {
    my ($ctx, $args, $cond) = @_;
    my $count = $ctx->stash('counter_count');
    
    return (defined($count) && ($count % 2 == 0));
}

sub hdlr_counter_if_mod {
    my ($ctx, $args, $cond) = @_;
    my $value = $args->{value} || 1;
    my $mod = $args->{mod} || 0;
    my $count = $ctx->stash('counter_count');
    
    return (defined($count) && ($count % $value == $mod));
}
